package com.qmth.distributed.print.api.mark;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.dto.mark.mark.MarkGroupDto;
import com.qmth.teachcloud.mark.dto.mark.mark.MarkSettingDto;
import com.qmth.teachcloud.mark.dto.mark.mark.MarkStatusDto;
import com.qmth.teachcloud.mark.dto.mark.mark.MarkTaskDto;
import com.qmth.teachcloud.mark.params.MarkResult;
import com.qmth.teachcloud.mark.service.MarkService;

import io.swagger.annotations.Api;

/**
 * <p>
 * 评卷端接口 前端控制器
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Api(tags = "评卷-评卷接口")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/mark")
public class MarkController {

	@Autowired
	private MarkService markService;

	@RequestMapping(value = "/getSetting", method = RequestMethod.POST)
	public MarkSettingDto getSetting(@RequestParam Long examId, @RequestParam String paperNumber,
			@RequestParam Integer groupNumber) {
		SysUser user = (SysUser) ServletUtil.getRequestUser();
		return markService.getSetting(user.getId(), examId, paperNumber, groupNumber);
	}

	@RequestMapping(value = "/getStatus", method = RequestMethod.POST)
	public MarkStatusDto status(@RequestParam Long examId, @RequestParam String paperNumber,
			@RequestParam Integer groupNumber) {
		SysUser user = (SysUser) ServletUtil.getRequestUser();
		return markService.getStatus(user.getId(), examId, paperNumber, groupNumber);
	}

	@RequestMapping(value = "/getGroup", method = RequestMethod.POST)
	@ResponseBody
	public List<MarkGroupDto> getGroup(@RequestParam Long examId, @RequestParam String paperNumber) {
		SysUser user = (SysUser) ServletUtil.getRequestUser();
		return markService.getGroup(user.getId(), examId, paperNumber);
	}

	@RequestMapping(value = "/getTask", method = RequestMethod.POST)
	public MarkTaskDto getTask(@RequestParam Long examId, @RequestParam String paperNumber,
			@RequestParam Integer groupNumber) {
		SysUser user = (SysUser) ServletUtil.getRequestUser();
		return markService.getTask(user.getId(), examId, paperNumber);
	}

	@RequestMapping(value = "/saveTask")
	public void saveTask(HttpServletRequest request, @RequestBody MarkResult markResult) {
		SysUser user = (SysUser) ServletUtil.getRequestUser();
	}

	@RequestMapping(value = "/getHistory", method = RequestMethod.POST)
	public List<MarkTaskDto> getHistory(HttpServletRequest request, @RequestParam int pageNumber,
			@RequestParam int pageSize, @RequestParam String order, @RequestParam String sort,
			@RequestParam(required = false) String secretNumber, @RequestParam(required = false) Double markerScore)
			throws Exception {
		SysUser user = (SysUser) ServletUtil.getRequestUser();
		return markService.getHistory(user.getId());
	}

	@RequestMapping(value = "/updateSetting", method = RequestMethod.POST)
	public void updateSetting(HttpServletRequest request, @RequestParam(required = false) String uiSetting,
			@RequestParam(required = false) String mode) {
		SysUser user = (SysUser) ServletUtil.getRequestUser();
	}

	@RequestMapping("/clear")
	public void clear(HttpServletRequest request) {
		SysUser user = (SysUser) ServletUtil.getRequestUser();
		markService.clear(user.getId());
	}
}
