package com.qmth.teachcloud.task.service.impl;

import com.qmth.distributed.print.business.bean.dto.initMarkData.ExamDetailCourseInitMarkDto;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.util.ExamTaskUtil;
import com.qmth.teachcloud.mark.bean.vo.parseCard.Struct;
import com.qmth.distributed.print.business.entity.ExamCard;
import com.qmth.distributed.print.business.entity.ExamStudent;
import com.qmth.distributed.print.business.entity.ExamTaskDetail;
import com.qmth.distributed.print.business.service.ExamCardService;
import com.qmth.distributed.print.business.service.ExamStudentService;
import com.qmth.distributed.print.business.service.ExamTaskDetailService;
import com.qmth.teachcloud.mark.utils.CardParseUtils;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkQuestion;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.entity.ScanAnswerCard;
import com.qmth.teachcloud.mark.enums.CardSource;
import com.qmth.teachcloud.mark.enums.QuestionType;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.FileStoreUtils;
import com.qmth.teachcloud.task.service.PrintFinishService;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Service
public class PrintFinishServiceImpl implements PrintFinishService {

    private static final Logger log = LoggerFactory.getLogger(PrintFinishServiceImpl.class);

    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private ScanAnswerCardService scanAnswerCardService;
    @Resource
    private ScanPaperService scanPaperService;
    @Resource
    private ExamStudentService examStudentService;
    @Resource
    private ExamTaskDetailService examTaskDetailService;
    @Resource
    private ExamCardService examCardService;
    @Resource
    private FileStoreUtils fileStoreUtils;
    @Resource
    private MarkFileService markFileService;

    /**
     * 打印结束后，生成阅卷科目数据
     *
     * @param examId
     * @param courseCode
     * @param courseName
     * @param paperNumber
     * @param packageCode
     */
    @Override
    public void insertMarkPaper(Long examId, String courseCode, String courseName, String paperNumber, String packageCode) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (markPaper != null) {
            return;
        }
        MarkPaper insertMarkPaper = new MarkPaper(examId, courseCode, courseName, paperNumber, packageCode);
        markPaperService.save(insertMarkPaper);
    }

    @Override
    public void insertMarkStudent(ExamDetailCourseInitMarkDto dto) {
        try {
            List<MarkStudent> markStudentList = new ArrayList<>();
            List<ExamStudent> examStudentList = examStudentService.listByExamDetailCourseIdNotIntMarkStudent(dto.getId());
            Set<String> secretNumberSet = new HashSet<>();
            for (ExamStudent examStudent : examStudentList) {
                MarkStudent markStudent = markStudentService.getById(examStudent.getId());
                if (markStudent != null) {
                    continue;
                }
                MarkStudent student = new MarkStudent(examStudent.getId(), dto.getExamId(), dto.getCourseCode(), dto.getCourseName(), dto.getPaperNumber(), examStudent.getStudentCode(), examStudent.getStudentName(), dto.getPackageCode(), dto.getExamPlace(), dto.getExamRoom(), examStudent.getCollegeName(), examStudent.getStudentName());
                if (student.getSecretNumber() == null) {
                    student.randomSecretNumber();
                    while (secretNumberSet.contains(student.getSecretNumber())
                            || markStudentService.countByExamIdAndSecretNumber(student.getExamId(), student.getSecretNumber()) > 0) {
                        student.randomSecretNumber();
                    }
                }
                markStudentList.add(student);
            }
            if (CollectionUtils.isNotEmpty(markStudentList)) {
                markStudentService.saveBatch(markStudentList);
            }
        } catch (Exception e) {
            log.info("考生同步阅卷失败");
        }
    }

    @Override
    public void insertScanAnswerCard(ExamDetailCourseInitMarkDto dto) {
        String paperType = dto.getPaperType();
        if (StringUtils.isNotBlank(paperType)) {
            ExamTaskDetail examTaskDetail = examTaskDetailService.findByExamIdAndCourseCodeAndPaperNumber(dto.getSchoolId(), dto.getExamId(), dto.getCourseCode(), dto.getPaperNumber());
            List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetail.getPaperAttachmentIds(), paperType);
            for (PaperInfoVo paperInfoVo : paperInfoVoList) {
                ExamCard examCard = examCardService.getById(paperInfoVo.getCardId());
                if (examCard != null) {
                    // 解析卡格式中试卷结构并保存
                    this.insertMarkQuestion(dto.getExamId(), dto.getPaperNumber(), examCard.getContent());

                    ScanAnswerCard scanAnswerCard = scanAnswerCardService.getByExamIdAndCoursePaperIdAndCardId(dto.getExamId(), dto.getCoursePaperId(), paperInfoVo.getCardId());
                    if (scanAnswerCard != null) {
                        continue;
                    }

                    File tempFile = SystemConstant.getFileTempDirVar(System.currentTimeMillis() + File.separator + SystemConstant.getNanoId(), SystemConstant.TEMP_PREFIX);
                    String rootPath = tempFile.getParent();
                    String jsonPath = rootPath + File.separator + paperInfoVo.getCardId() + SystemConstant.JSON_PREFIX;
                    File jsonFile = SystemConstant.createJsonFile(jsonPath, examCard.getContent());
                    if (jsonFile.exists()) {
                        try {
                            FileInputStream inputStream = new FileInputStream(jsonFile);
                            String md5 = DigestUtils.md5Hex(inputStream);
                            Integer number = scanAnswerCardService.findMaxCardNumberByExamId(dto.getExamId());
                            String path = markFileService.getAnswerCardUri(dto.getExamId(), dto.getCoursePaperId(), number);
                            String uploadPath = fileStoreUtils.uploadFile(inputStream, md5, UploadFileEnum.CARD, path);

                            scanAnswerCard = new ScanAnswerCard();
                            scanAnswerCard.setExamId(dto.getExamId());
                            scanAnswerCard.setPaperNumber(dto.getPaperNumber());
                            scanAnswerCard.setCoursePaperId(dto.getCoursePaperId());
                            scanAnswerCard.setCardId(paperInfoVo.getCardId());
                            scanAnswerCard.setNumber(number);
                            scanAnswerCard.setSource(CardSource.WEB);
                            scanAnswerCard.setUri(uploadPath);
                            scanAnswerCard.setMd5(md5);
                            scanAnswerCard.setNeedAdapte(true);
                            scanAnswerCardService.save(scanAnswerCard);
                        } catch (IOException e) {
                            log.info("考试[{}]，试卷编号[{}]，题卡ID[{}]同步阅卷失败", dto.getExamId(), dto.getPaperNumber(), paperInfoVo.getCardId());
                        } finally {
                            try {
                                FileUtils.forceDeleteOnExit(jsonFile);
                            } catch (IOException e) {
                                throw new RuntimeException(e);
                            }
                        }
                    }
                }
            }
        }
    }

    @Override
    public void insertMarkQuestion(Long examId, String paperNumber, String content) {
        List<Struct> structList = CardParseUtils.parseCardContent(content);
        if (CollectionUtils.isEmpty(structList)) {
            return;
        }
        List<MarkQuestion> markQuestionList = markQuestionService.listQuestionByExamIdAndPaperNumber(examId, paperNumber);
        if (CollectionUtils.isEmpty(markQuestionList)) {
            List<MarkQuestion> markQuestions = new ArrayList<>();
            for (Struct struct : structList) {
                MarkQuestion markQuestion = new MarkQuestion();
                markQuestion.setId(SystemConstant.getDbUuid());
                markQuestion.setExamId(examId);
                markQuestion.setPaperNumber(paperNumber);
                markQuestion.setObjective(struct.getObjective());
                markQuestion.setMainNumber(struct.getMainNumber());
                markQuestion.setSubNumber(struct.getSubNumber());
                markQuestion.setMainTitle(struct.getMainTitle());
                markQuestion.setQuestionType(QuestionType.findByValue(struct.getType()));
                markQuestion.setPaperIndex(struct.getPaperIndex());
                markQuestion.setPageIndex(struct.getPageIndex());
                markQuestions.add(markQuestion);
            }
            markQuestionService.saveBatch(markQuestions);
        } else {

            List<ScanAnswerCard> scanAnswerCardList = scanAnswerCardService.listByExamIdAndPaperNumber(examId, paperNumber);
            if (CollectionUtils.isNotEmpty(scanAnswerCardList)) {
                for (ScanAnswerCard scanAnswerCard : scanAnswerCardList) {
                    int count = scanPaperService.getCountByExamAndCardNumber(examId, scanAnswerCard.getNumber());
                    if (count > 0) return;
                }
            }

            // todo 题卡变动后，结构数据处理逻辑
            if (structList.size() == markQuestionList.size()) {
                for (int i = 0; i < structList.size(); i++) {
                    Struct struct = structList.get(i);
                    MarkQuestion markQuestion = markQuestionList.get(i);
                    if (markQuestion.getGroupNumber() != null) {
                        if (struct.getMainNumber() == markQuestion.getMainNumber()
                                && struct.getSubNumber() == markQuestion.getSubNumber()
                                && struct.getObjective().equals(markQuestion.getObjective())
                                && struct.getType() == markQuestion.getQuestionType().getValue()
                                && struct.getMainTitle().equals(markQuestion.getMainTitle())) {
                            continue;
                        }
                    }
                }
            } else {

            }
        }
    }
}
